<?php
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class CusminURLShortener
{
    const SLUG = 'cusmin-url-shortener';
    const PLUGIN_NAME = 'Cusmin URL Shortener';

    private $options;

    public function __construct(){
        if(!is_admin()){
            return;
        }

        $this->loadOptions();
        $this->registerHooks();
    }

    public function registerScripts(){
        wp_register_script( self::SLUG, plugins_url( '/js/'.self::SLUG.'.js', __FILE__ ), array( 'jquery' ) );
        wp_register_style( self::SLUG, plugins_url( '/css/'.self::SLUG.'.css', __FILE__ ) );

        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( self::SLUG );
        wp_enqueue_style( self::SLUG );
    }

    public function ajax_short(){

        if(!is_user_logged_in()){
            die;
        }

        if ( !wp_verify_nonce( $_REQUEST['nonce'], "cusmin_us_short_nonce")) {
            exit("No naughty business please");
        }

        if(empty($_REQUEST["url"])){
            die('URL is not set');
        }

        $url = $_REQUEST["url"];

        $sh = new CusminGoogl($this->getGoogleAPIKey());
        if(!empty($_REQUEST['expand'])){
            $url = $sh->expand($url);
        }else{
            $url = $sh->shorten($url);
        }

        if(empty($url)){
            return wp_send_json(['status' => 'error', 'message' => 'Check API Key']);
        }

        return wp_send_json(['status' => 'success', 'url' => $url]);

        die();
    }

    public function ajax_expand(){
        $_REQUEST['expand'] = 'true';
        $this->ajax_short();
    }

    public function footer_scripts(){
        wp_nonce_field( 'cusmin_us_short_nonce', 'cusmin_us_short_nonce' );
    }

    public function registerAjax(){
        add_action("wp_ajax_cusmin_us_short", array($this, "ajax_short"));
        add_action("wp_ajax_nopriv_cusmin_us_short", array($this, "ajax_short"));

        add_action("wp_ajax_cusmin_us_expand", array($this, "ajax_expand"));
        add_action("wp_ajax_nopriv_cusmin_us_expand", array($this, "ajax_expand"));
    }

    private function loadOptions(){
        $this->options = get_option( self::SLUG );
        if(!$this->options){
            $this->options = [];
        }
    }

    private function getGoogleAPIKey(){
        return esc_attr($this->options['google-api-key']);
    }

    private function getAdminBarShortener(){
        return $this->options['admin-bar-shortener'];
    }

    private function getUndoShortening(){
        return $this->options['shorten-undo'];
    }

    public function settingsPage(){
        ?>
        <div class="wrap">
            <div class="cusmin-us-logo"></div>
            <h1><?php echo self::PLUGIN_NAME; ?> Settings&nbsp;<a target="_blank" href="https://cusmin.com/url-shortener?ref=cus-title" class="button button-secondary">Help</a></h1>

            <form method="post" action="options.php">
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row"><h2>Settings</h2></th>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Hide Admin Bar shortener</th>
                        <td><input type="checkbox" name="<?php echo self::SLUG; ?>[admin-bar-shortener]" value="1"<?php checked( $this->getAdminBarShortener() ); ?>  /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Disable reverse shortening</th>
                        <td><input type="checkbox" name="<?php echo self::SLUG; ?>[shorten-undo]" value="1"<?php checked( $this->getUndoShortening() ); ?>  /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><h2>Google URL Shortener</h2></th>
                    </tr>
                    <tr valign="top">
                        <th scope="row">API KEY (<a target="_blank" href="https://console.developers.google.com/apis/library/urlshortener.googleapis.com?q=shortener">get your key</a>)</th>
                        <td><input style="width: 330px" type="text" name="<?php echo self::SLUG; ?>[google-api-key]" value="<?php echo $this->getGoogleAPIKey(); ?>" /></td>
                    </tr>
                </table>
                <?php
                // This prints out all hidden setting fields
                settings_fields( self::SLUG.'-group' );
                do_settings_sections( self::SLUG.'-setting-admin' );
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    function admin_menu() {
        add_options_page(
            __( self::PLUGIN_NAME, self::SLUG ),
            __( self::PLUGIN_NAME, self::SLUG ),
            'manage_options',
            self::SLUG,
            array(
                $this,
                'settingsPage'
            )
        );
    }

    function admin_bar_menu(\WP_Admin_Bar $bar){
        $bar->add_menu( array(
            'id'     => 'cusmin-us-ab',
            'parent' => 'top-secondary',
            'group'  => null,
            'title'  => __( '<input placeholder="Shorten URL" type="text" class="ab-cusmin-input"/><div class="cusmin-go-shorten">GO</div>', self::SLUG ),
            'href'   => '#',
            'meta'   => array(
                'class'    => 'cusmin-us-ab--item',
                'tabindex' => 0,
            ),
        ) );
    }

    public function onAdminInit(){
        $this->registerScripts();
        $this->registerAjax();

        register_setting( self::SLUG.'-group', self::SLUG );
    }

    public function pluginLinks( $links ) {
        $newLinks = array(
            '<a href="options-general.php?page='.self::SLUG.'" >' . __( 'Settings' ) . '</a>',
            '<a target="_blank" href="https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=url.shortener@cusmin.com&item_name=Support+for+Cusmin+URL+Shortener+Plugin+Development" >' . __( 'Donate' ) . '</a>',
            '<a target="_blank" href="https://cusmin.com/url-shortener?ref=plugins-page" >' . __( 'Help' ) . '</a>'
        );
        return array_merge($newLinks, $links);
    }

    public function disableShorteningBodyClass($classes){
        $classes .= ' '.self::SLUG.'-disable-undo ';
        return $classes;
    }

    private function registerHooks(){

        if(!$this->getAdminBarShortener()){
            add_action('admin_bar_menu', array( $this, 'admin_bar_menu' ) );
        }
        if($this->getUndoShortening()){
            add_filter( 'admin_body_class', array($this, 'disableShorteningBodyClass') );
        }
        add_action('admin_menu', array( $this, 'admin_menu' ) );
        add_action('admin_init', array($this, 'onAdminInit'));
        add_action('admin_footer', array($this, "footer_scripts"));
        add_filter("plugin_action_links_".self::SLUG.'/index.php', array($this, 'pluginLinks') );
    }
}